@@ -1,5 +1,5 @@
 /*
- * "$Id: snmp.c 10535 2012-06-22 03:45:53Z mike $"
+ * "$Id: snmp.c 7810 2008-07-29 01:11:15Z mike $"
  *
  *   SNMP discovery backend for CUPS.
  *
@@ -1025,6 +1025,11 @@ read_snmp_response(int fd)		/* I - SNMP socket file descriptor */
 	    * Description is the IEEE-1284 device ID...
 	    */
 
+            char *ptr;			/* Pointer into device ID */
+
+            for (ptr = (char *)packet.object_value.string.bytes; *ptr; ptr ++)
+              if (*ptr == '\n')
+                *ptr = ';';		/* A lot of bad printers put a newline */
 	    if (!device->id)
 	      device->id = strdup((char *)packet.object_value.string.bytes);
 
@@ -1066,8 +1071,11 @@ read_snmp_response(int fd)		/* I - SNMP socket file descriptor */
 	  */
 
 	  char	make_model[256];	/* Make and model */
+          char *ptr;			/* Pointer into device ID */
 
-
+          for (ptr = (char *)packet.object_value.string.bytes; *ptr; ptr ++)
+            if (*ptr == '\n')
+              *ptr = ';';		/* A lot of bad printers put a newline */
 	  if (device->id)
 	    free(device->id);
 
@@ -1383,5 +1391,5 @@ update_cache(snmp_cache_t *device,	/* I - Device */
 
 
 /*
- * End of "$Id: snmp.c 10535 2012-06-22 03:45:53Z mike $".
+ * End of "$Id: snmp.c 7810 2008-07-29 01:11:15Z mike $".
  */
