@@ -1,5 +1,5 @@
 /*
- * "$Id: printers.c 10431 2012-04-23 19:19:19Z mike $"
+ * "$Id: printers.c 7940 2008-09-16 00:45:16Z mike $"
  *
  *   Printer status CGI for CUPS.
  *
@@ -397,7 +397,7 @@ show_all_printers(http_t     *http,	/* I - Connection to server */
     sprintf(val, "%d", count);
     cgiSetVariable("TOTAL", val);
 
-    if ((var = cgiGetVariable("ORDER")) != NULL)
+    if ((var = cgiGetVariable("ORDER")) != NULL && *var)
       ascending = !_cups_strcasecmp(var, "asc");
     else
       ascending = 1;
@@ -574,5 +574,5 @@ show_printer(http_t     *http,		/* I - Connection to server */
 
 
 /*
- * End of "$Id: printers.c 10431 2012-04-23 19:19:19Z mike $".
+ * End of "$Id: printers.c 7940 2008-09-16 00:45:16Z mike $".
  */
