@@ -1,9 +1,9 @@
 /*
- * "$Id: ipp-private.h 6649 2007-07-11 21:46:42Z mike $"
+ * "$Id: ipp-private.h 7259 2008-01-28 22:26:04Z mike $"
  *
- *   Private IPP definitions for the Common UNIX Printing System (CUPS).
+ *   Private IPP definitions for CUPS.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -22,7 +22,7 @@
  * Include necessary headers...
  */
 
-#  include "ipp.h"
+#  include <cups/ipp.h>
 
 
 /*
@@ -34,15 +34,26 @@ extern "C" {
 #  endif /* __cplusplus */
 
 
+/*
+ * Constants...
+ */
+
+#  define IPP_BUF_SIZE	(IPP_MAX_LENGTH + 2)
+					/* Size of buffer */
+
+
 /*
  * Structures...
  */
 
 typedef struct				/**** Attribute mapping data ****/
 {
+  int		multivalue;		/* Option has multiple values? */
   const char	*name;			/* Option/attribute name */
   ipp_tag_t	value_tag;		/* Value tag for this attribute */
   ipp_tag_t	group_tag;		/* Group tag for this attribute */
+  ipp_tag_t	alt_group_tag;		/* Alternate group tag for this
+					 * attribute */
 } _ipp_option_t;
 
 
@@ -50,9 +61,7 @@ typedef struct				/**** Attribute mapping data ****/
  * Prototypes for private functions...
  */
 
-extern ipp_attribute_t	*_ippAddAttr(ipp_t *, int);
 extern _ipp_option_t	*_ippFindOption(const char *name);
-extern void		_ippFreeAttr(ipp_attribute_t *);
 
 
 /*
@@ -65,5 +74,5 @@ extern void		_ippFreeAttr(ipp_attribute_t *);
 #endif /* !_CUPS_IPP_H_ */
 
 /*
- * End of "$Id: ipp-private.h 6649 2007-07-11 21:46:42Z mike $".
+ * End of "$Id: ipp-private.h 7259 2008-01-28 22:26:04Z mike $".
  */
