@@ -1,9 +1,9 @@
 /*
- * "$Id: sidechannel.c 8827 2009-09-22 23:20:35Z mike $"
+ * "$Id: sidechannel.c 7720 2008-07-11 22:46:21Z mike $"
  *
- *   Side-channel API code for the Common UNIX Printing System (CUPS).
+ *   Side-channel API code for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -29,14 +29,12 @@
  */
 
 #include "sidechannel.h"
-#include "string.h"
-#include "debug.h"
+#include "cups-private.h"
 #ifdef WIN32
 #  include <io.h>
 #else
 #  include <unistd.h>
 #endif /* WIN32 */
-#include <errno.h>
 #ifdef __hpux
 #  include <sys/time.h>
 #elif !defined(WIN32)
@@ -46,10 +44,18 @@
 #  include <sys/time.h>
 #endif /* !WIN32 */
 #ifdef HAVE_POLL
-#  include <sys/poll.h>
+#  include <poll.h>
 #endif /* HAVE_POLL */
 
 
+/*
+ * Buffer size for side-channel requests...
+ */
+
+#define _CUPS_SC_MAX_DATA	65535
+#define _CUPS_SC_MAX_BUFFER	65540
+
+
 /*
  * 'cupsSideChannelDoRequest()' - Send a side-channel command to a backend and wait for a response.
  *
@@ -63,7 +69,7 @@
  * pointed to by the "data" parameter.  cupsSideChannelDoRequest() will
  * update the value to contain the number of data bytes in the buffer.
  *
- * @since CUPS 1.3/Mac OS X 10.5@
+ * @since CUPS 1.3/OS X 10.5@
  */
 
 cups_sc_status_t			/* O  - Status of command */
@@ -102,7 +108,7 @@ cupsSideChannelDoRequest(
  * pointed to by the "data" parameter.  cupsSideChannelDoRequest() will
  * update the value to contain the number of data bytes in the buffer.
  *
- * @since CUPS 1.3/Mac OS X 10.5@
+ * @since CUPS 1.3/OS X 10.5@
  */
 
 int					/* O - 0 on success, -1 on error */
@@ -113,9 +119,10 @@ cupsSideChannelRead(
     int               *datalen,		/* IO - Size of data buffer on entry, number of bytes in buffer on return */
     double            timeout)		/* I  - Timeout in seconds */
 {
-  char		buffer[16388];		/* Message buffer */
+  char		*buffer;		/* Message buffer */
   int		bytes;			/* Bytes read */
   int		templen;		/* Data length from message */
+  int		nfds;			/* Number of file descriptors */
 #ifdef HAVE_POLL
   struct pollfd	pfd;			/* Poll structure for poll() */
 #else /* select() */
@@ -143,39 +150,32 @@ cupsSideChannelRead(
   pfd.fd     = CUPS_SC_FD;
   pfd.events = POLLIN;
 
-  if (timeout < 0.0)
-  {
-    if (poll(&pfd, 1, -1) < 1)
-      return (-1);
-  }
-  else if (poll(&pfd, 1, (long)(timeout * 1000)) < 1)
-    return (-1);
+  while ((nfds = poll(&pfd, 1,
+		      timeout < 0.0 ? -1 : (long)(timeout * 1000))) < 0 &&
+	 (errno == EINTR || errno == EAGAIN))
+    ;
 
 #else /* select() */
   FD_ZERO(&input_set);
   FD_SET(CUPS_SC_FD, &input_set);
 
-  if (timeout < 0.0)
-  {
-    if (select(CUPS_SC_FD + 1, &input_set, NULL, NULL, NULL) < 1)
-    {
-      DEBUG_printf(("1cupsSideChannelRead: Select error: %s", strerror(errno)));
-      return (-1);
-    }
-  }
-  else
-  {
-    stimeout.tv_sec  = (int)timeout;
-    stimeout.tv_usec = (int)(timeout * 1000000) % 1000000;
+  stimeout.tv_sec  = (int)timeout;
+  stimeout.tv_usec = (int)(timeout * 1000000) % 1000000;
+
+  while ((nfds = select(CUPS_SC_FD + 1, &input_set, NULL, NULL,
+			timeout < 0.0 ? NULL : &stimeout)) < 0 &&
+	 (errno == EINTR || errno == EAGAIN))
+    ;
 
-    if (select(CUPS_SC_FD + 1, &input_set, NULL, NULL, &stimeout) < 1)
-    {
-      DEBUG_puts("1cupsSideChannelRead: Select timeout");
-      return (-1);
-    }
-  }
 #endif /* HAVE_POLL */
 
+  if (nfds < 1)
+  {
+    *command = CUPS_SC_CMD_NONE;
+    *status  = nfds==0 ? CUPS_SC_STATUS_TIMEOUT : CUPS_SC_STATUS_IO_ERROR;
+    return (-1);
+  }
+
  /*
   * Read a side-channel message for the format:
   *
@@ -183,16 +183,28 @@ cupsSideChannelRead(
   * -------  -------------------------------------------
   * 0        Command code
   * 1        Status code
-  * 2-3      Data length (network byte order) <= 16384
+  * 2-3      Data length (network byte order)
   * 4-N      Data
   */
 
-  while ((bytes = read(CUPS_SC_FD, buffer, sizeof(buffer))) < 0)
+  if ((buffer = _cupsBufferGet(_CUPS_SC_MAX_BUFFER)) == NULL)
+  {
+    *command = CUPS_SC_CMD_NONE;
+    *status  = CUPS_SC_STATUS_TOO_BIG;
+
+    return (-1);
+  }
+
+  while ((bytes = read(CUPS_SC_FD, buffer, _CUPS_SC_MAX_BUFFER)) < 0)
     if (errno != EINTR && errno != EAGAIN)
     {
       DEBUG_printf(("1cupsSideChannelRead: Read error: %s", strerror(errno)));
+
+      _cupsBufferRelease(buffer);
+
       *command = CUPS_SC_CMD_NONE;
       *status  = CUPS_SC_STATUS_IO_ERROR;
+
       return (-1);
     }
 
@@ -203,8 +215,12 @@ cupsSideChannelRead(
   if (bytes < 4)
   {
     DEBUG_printf(("1cupsSideChannelRead: Short read of %d bytes", bytes));
+
+    _cupsBufferRelease(buffer);
+
     *command = CUPS_SC_CMD_NONE;
     *status  = CUPS_SC_STATUS_BAD_MESSAGE;
+
     return (-1);
   }
 
@@ -213,11 +229,15 @@ cupsSideChannelRead(
   */
 
   if (buffer[0] < CUPS_SC_CMD_SOFT_RESET ||
-      buffer[0] > CUPS_SC_CMD_SNMP_GET_NEXT)
+      buffer[0] >= CUPS_SC_CMD_MAX)
   {
     DEBUG_printf(("1cupsSideChannelRead: Bad command %d!", buffer[0]));
+
+    _cupsBufferRelease(buffer);
+
     *command = CUPS_SC_CMD_NONE;
     *status  = CUPS_SC_STATUS_BAD_MESSAGE;
+
     return (-1);
   }
 
@@ -260,6 +280,8 @@ cupsSideChannelRead(
     memcpy(data, buffer + 4, templen);
   }
 
+  _cupsBufferRelease(buffer);
+
   DEBUG_printf(("1cupsSideChannelRead: Returning status=%d", *status));
 
   return (0);
@@ -286,7 +308,7 @@ cupsSideChannelRead(
  * support SNMP queries.  @code CUPS_SC_STATUS_NO_RESPONSE@ is returned when
  * the printer does not respond to the SNMP query.
  *
- * @since CUPS 1.4/Mac OS X 10.6@ 
+ * @since CUPS 1.4/OS X 10.6@
  */
 
 cups_sc_status_t			/* O  - Query status */
@@ -298,7 +320,7 @@ cupsSideChannelSNMPGet(
 {
   cups_sc_status_t	status;		/* Status of command */
   cups_sc_command_t	rcommand;	/* Response command */
-  char			real_data[2048];/* Real data buffer for response */
+  char			*real_data;	/* Real data buffer for response */
   int			real_datalen,	/* Real length of data buffer */
 			real_oidlen;	/* Length of returned OID string */
 
@@ -324,12 +346,21 @@ cupsSideChannelSNMPGet(
                            (int)strlen(oid) + 1, timeout))
     return (CUPS_SC_STATUS_TIMEOUT);
 
-  real_datalen = sizeof(real_data);
+  if ((real_data = _cupsBufferGet(_CUPS_SC_MAX_BUFFER)) == NULL)
+    return (CUPS_SC_STATUS_TOO_BIG);
+
+  real_datalen = _CUPS_SC_MAX_BUFFER;
   if (cupsSideChannelRead(&rcommand, &status, real_data, &real_datalen, timeout))
+  {
+    _cupsBufferRelease(real_data);
     return (CUPS_SC_STATUS_TIMEOUT);
+  }
 
   if (rcommand != CUPS_SC_CMD_SNMP_GET)
+  {
+    _cupsBufferRelease(real_data);
     return (CUPS_SC_STATUS_BAD_MESSAGE);
+  }
 
   if (status == CUPS_SC_STATUS_OK)
   {
@@ -341,7 +372,10 @@ cupsSideChannelSNMPGet(
     real_datalen -= real_oidlen;
 
     if ((real_datalen + 1) > *datalen)
+    {
+      _cupsBufferRelease(real_data);
       return (CUPS_SC_STATUS_TOO_BIG);
+    }
 
     memcpy(data, real_data + real_oidlen, real_datalen);
     data[real_datalen] = '\0';
@@ -349,6 +383,8 @@ cupsSideChannelSNMPGet(
     *datalen = real_datalen;
   }
 
+  _cupsBufferRelease(real_data);
+
   return (status);
 }
 
@@ -378,7 +414,7 @@ cupsSideChannelSNMPGet(
  * support SNMP queries.  @code CUPS_SC_STATUS_NO_RESPONSE@ is returned when
  * the printer does not respond to the first SNMP query.
  *
- * @since CUPS 1.4/Mac OS X 10.6@ 
+ * @since CUPS 1.4/OS X 10.6@
  */
 
 cups_sc_status_t			/* O - Status of first query of @code CUPS_SC_STATUS_OK@ on success */
@@ -390,7 +426,7 @@ cupsSideChannelSNMPWalk(
 {
   cups_sc_status_t	status;		/* Status of command */
   cups_sc_command_t	rcommand;	/* Response command */
-  char			real_data[2048];/* Real data buffer for response */
+  char			*real_data;	/* Real data buffer for response */
   int			real_datalen,	/* Real length of data buffer */
 			real_oidlen,	/* Length of returned OID string */
 			oidlen;		/* Length of first OID */
@@ -408,6 +444,9 @@ cupsSideChannelSNMPWalk(
   if (!oid || !*oid || !cb)
     return (CUPS_SC_STATUS_BAD_MESSAGE);
 
+  if ((real_data = _cupsBufferGet(_CUPS_SC_MAX_BUFFER)) == NULL)
+    return (CUPS_SC_STATUS_TOO_BIG);
+
  /*
   * Loop until the OIDs don't match...
   */
@@ -424,15 +463,24 @@ cupsSideChannelSNMPWalk(
 
     if (cupsSideChannelWrite(CUPS_SC_CMD_SNMP_GET_NEXT, CUPS_SC_STATUS_NONE,
                              current_oid, (int)strlen(current_oid) + 1, timeout))
+    {
+      _cupsBufferRelease(real_data);
       return (CUPS_SC_STATUS_TIMEOUT);
+    }
 
-    real_datalen = sizeof(real_data);
+    real_datalen = _CUPS_SC_MAX_BUFFER;
     if (cupsSideChannelRead(&rcommand, &status, real_data, &real_datalen,
                             timeout))
+    {
+      _cupsBufferRelease(real_data);
       return (CUPS_SC_STATUS_TIMEOUT);
+    }
 
     if (rcommand != CUPS_SC_CMD_SNMP_GET_NEXT)
+    {
+      _cupsBufferRelease(real_data);
       return (CUPS_SC_STATUS_BAD_MESSAGE);
+    }
 
     if (status == CUPS_SC_STATUS_OK)
     {
@@ -447,6 +495,7 @@ cupsSideChannelSNMPWalk(
         * Done with this set of OIDs...
 	*/
 
+	_cupsBufferRelease(real_data);
         return (CUPS_SC_STATUS_OK);
       }
 
@@ -460,7 +509,7 @@ cupsSideChannelSNMPWalk(
       * Call the callback with the OID and data...
       */
 
-      (*cb)(real_data, real_data + real_oidlen, real_datalen, context); 
+      (*cb)(real_data, real_data + real_oidlen, real_datalen, context);
 
      /*
       * Update the current OID...
@@ -472,6 +521,8 @@ cupsSideChannelSNMPWalk(
   }
   while (status == CUPS_SC_STATUS_OK);
 
+  _cupsBufferRelease(real_data);
+
   return (status);
 }
 
@@ -482,7 +533,7 @@ cupsSideChannelSNMPWalk(
  * This function is normally only called by backend programs to send
  * responses to a filter, driver, or port monitor program.
  *
- * @since CUPS 1.3/Mac OS X 10.5@
+ * @since CUPS 1.3/OS X 10.5@
  */
 
 int					/* O - 0 on success, -1 on error */
@@ -493,7 +544,7 @@ cupsSideChannelWrite(
     int               datalen,		/* I - Number of bytes of data */
     double            timeout)		/* I - Timeout in seconds */
 {
-  char		buffer[16388];		/* Message buffer */
+  char		*buffer;		/* Message buffer */
   int		bytes;			/* Bytes written */
 #ifdef HAVE_POLL
   struct pollfd	pfd;			/* Poll structure for poll() */
@@ -507,8 +558,8 @@ cupsSideChannelWrite(
   * Range check input...
   */
 
-  if (command < CUPS_SC_CMD_SOFT_RESET || command > CUPS_SC_CMD_SNMP_GET_NEXT ||
-      datalen < 0 || datalen > 16384 || (datalen > 0 && !data))
+  if (command < CUPS_SC_CMD_SOFT_RESET || command >= CUPS_SC_CMD_MAX ||
+      datalen < 0 || datalen > _CUPS_SC_MAX_DATA || (datalen > 0 && !data))
     return (-1);
 
  /*
@@ -557,6 +608,9 @@ cupsSideChannelWrite(
   * 4-N      Data
   */
 
+  if ((buffer = _cupsBufferGet(datalen + 4)) == NULL)
+    return (-1);
+
   buffer[0] = command;
   buffer[1] = status;
   buffer[2] = datalen >> 8;
@@ -572,12 +626,17 @@ cupsSideChannelWrite(
 
   while (write(CUPS_SC_FD, buffer, bytes) < 0)
     if (errno != EINTR && errno != EAGAIN)
+    {
+      _cupsBufferRelease(buffer);
       return (-1);
+    }
+
+  _cupsBufferRelease(buffer);
 
   return (0);
 }
 
 
 /*
- * End of "$Id: sidechannel.c 8827 2009-09-22 23:20:35Z mike $".
+ * End of "$Id: sidechannel.c 7720 2008-07-11 22:46:21Z mike $".
  */
